#patches_base=v1.4.0
Patch0001: 0001-file_zbc-fixed-compile-error-under-ppc64le.patch
Patch0002: 0002-file_zbc-optionally-build-zbc-handler.patch

%global oversion 1.4.0

Name:		tcmu-runner
Version:	1.4.0
Release:	1%{?dist}
Summary:	A daemon that handles the userspace side of the LIO TCM-User backstore

Group:		System Environment/Daemons
License:	Apache 2.0 or LGPLv2.1
URL:		https://github.com/open-iscsi/tcmu-runner
Source0:	https://github.com/open-iscsi/${name}/archive/v%{oversion}/%{name}-v%{oversion}.tar.gz

Requires:	kmod
Requires:	libnl3
Requires:	librbd1 >= 2:12.2.4
Requires:	logrotate
Requires:	zlib

BuildRequires:	cmake
BuildRequires:	gcc
BuildRequires:	glib2-devel
BuildRequires:	kmod-devel
BuildRequires:	libnl3-devel
BuildRequires:	librbd1-devel >= 2:12.2.4
BuildRequires:	make
BuildRequires:	systemd-units
BuildRequires:	zlib-devel

Obsoletes:	libtcmu

%description
A daemon that handles the userspace side of the LIO TCM-User backstore.

LIO is the SCSI target in the Linux kernel. It is entirely kernel code, and
allows exported SCSI logical units (LUNs) to be backed by regular files or
block devices. But, if we want to get fancier with the capabilities of the
device we're emulating, the kernel is not necessarily the right place. While
there are userspace libraries for compression, encryption, and clustered
storage solutions like Ceph or Gluster, these are not accessible from the
kernel.

The TCMU userspace-passthrough backstore allows a userspace process to handle
requests to a LUN. But since the kernel-user interface that TCMU provides
must be fast and flexible, it is complex enough that we'd like to avoid each
userspace handler having to write boilerplate code.

tcmu-runner handles the messy details of the TCMU interface -- UIO, netlink,
pthreads, and DBus -- and exports a more friendly C plugin module API. Modules
using this API are called "TCMU handlers". Handler authors can write code just
to handle the SCSI commands as desired, and can also link with whatever
userspace libraries they like.

%prep
%autosetup -p1 -n %{name}-%{oversion}

%build
%cmake -DSUPPORT_SYSTEMD=ON -Dwith-glfs=false -Dwith-qcow=false -Dwith-zbc=false .
make %{?_smp_mflags}

%pre
if [ "$1" = "2" ]; then
  # RHEL kernel does not support runner restarting hung IO, so we must
  # stop rbd-target-gw to make sure running IO is completed and new IO
  # is not sent to us while runner is restarting.
  systemctl is-active --quiet rbd-target-gw && systemctl stop rbd-target-gw
  systemctl stop tcmu-runner
fi

%install
make install DESTDIR=%{buildroot}
mkdir -p %{buildroot}/etc/tcmu/
install -m 644 tcmu.conf %{buildroot}/etc/tcmu/
mkdir -p %{buildroot}%{_sysconfdir}/logrotate.d/
install -m 644 logrotate.conf %{buildroot}%{_sysconfdir}/logrotate.d/tcmu-runner

%post
/sbin/ldconfig
systemctl daemon-reload
systemctl start tcmu-runner
systemctl is-enabled --quiet rbd-target-gw && systemctl start rbd-target-gw

%files
%defattr(-,root,root)
%{_bindir}/tcmu-runner
%{_libdir}/*
%{_libdir}/tcmu-runner/*
%dir %{_sysconfdir}/dbus-1/
%dir %{_sysconfdir}/dbus-1/system.d
%config %{_sysconfdir}/dbus-1/system.d/tcmu-runner.conf
%dir %{_datadir}/dbus-1/
%dir %{_datadir}/dbus-1/system-services/
%{_datadir}/dbus-1/system-services/org.kernel.TCMUService1.service
%{_unitdir}/tcmu-runner.service
%dir %{_usr}/lib64/tcmu-runner/
%{_mandir}/man8/*
%doc README.md LICENSE.LGPLv2.1 LICENSE.Apache2
%dir %{_sysconfdir}/tcmu/
%config %{_sysconfdir}/tcmu/tcmu.conf
%attr(0600,-,-) %config(noreplace) %{_sysconfdir}/tcmu/tcmu.conf
%config(noreplace) %{_sysconfdir}/logrotate.d/tcmu-runner

%changelog
* Tue Oct 09 2018 Jason Dillaman <dillaman@redhat.com> 1.4.0-1
- rebased to 1.4.0 release (rhbz#1619839)
- file_zbc: optionally build zbc handler
- file_zbc: fixed compile error under ppc64le

* Fri Aug 31 2018 Jason Dillaman <dillaman@redhat.com> 1.4.0-0.3
- runner: reopen dev right away if we know we had the lock and lost it (rhbz#1623737)
- runner: fix double log message (rhbz#1623737)
- runner: fix double close/free (rhbz#1623737)
- tcmur:fix the initial value of the return var (rhbz#1623737)
- Do not return alua state transition (rhbz#1623737)
- runner/rbd: update lock state on RTPG/INQUIRY (rhbz#1623737)
- alua: reduce the debug logs (rhbz#1623737)

* Tue Aug 14 2018 Jason Dillaman <dillaman@redhat.com> 1.4.0-0.2
- rbd: detect pools associated to NVMe device class (rhbz#1508611)

* Sat May 5 2018 Jason Dillaman <dillaman@redhat.com> 1.4.0-0.1.0
- rebased to 1.4.0-rc1

* Fri Apr 27 2018 Jason Dillaman <dillaman@redhat.com> 1.3.0-3
- merged libtcmu into tcmu-runner package

* Thu Apr 26 2018 Jason Dillaman <dillaman@redhat.com> 1.3.0-2
- tcmur: fix ret value in tcmur_dev_update_size
- fix the condition of sleep in _tcmu_reopen_dev
- config: reset to default values after the options being set
- tcmu-runner: improve the logging always to get daemon's exit state
- glfs: reopen on connection lost
- configfs: remove null chars in the string readed from the sys files
- Set correct sense upon getting write to FULL zone
- Add installation action for file_zbc handler
- zbc: fix maximum number of open zones
- zbc: fix write emulation
- Fixed build procedure in README.
- logging: improve log details
- glfs: remove size checks in handler
- extra: Add make_runnerrpms.sh support
- handler: do not perform implicit transition if command not supported
- glfs: update the glfs config descritions and add usage of creation

* Thu Apr 26 2018 Jason Dillaman <dillaman@redhat.com> 1.3.0-1
- rebased to 1.3.0
- spec changes to not overwrite tcmu.conf
- spec changes to stop/start services before/after installation

* Thu Sep 28 2017 Jason Dillaman <dillaman@redhat.com> 1.3.0-0.4
- rebased to 1.3.0-rc4

* Sat Sep 09 2017 Jason Dillaman <dillaman@redhat.com> 1.3.0-0.2.4
- libtcmu: fix name check for nl commands

* Tue Sep 05 2017 Jason Dillaman <dillaman@redhat.com> 1.3.0-0.2.3
- libtcmu: move genl_unregister_family to teardown_netlink
- open_handlers: fix dlsym return
- rbd: try to remain cluster's osd op timeout default value
- libtcmu: handle error when netlink family group is not found
- log config: fix tcmu_conf_set_options
- config: fix the default log level in tcmu.conf
- logger: logdir path setting via Env variable 'TCMU_LOGDIR'
- logdir: make log dir configurable in tcmu.conf
- logdir: fix error when making recursion directories
- logdir: move tcmu_logdir_create to log file and make it globally
- logdir: make tcmu_log_dir as static
- README: update and add logger setting and system config sections
- tcmur: fixed possible uninitialized variable warning
- fbo: use get be helpers.
- file_optical: fix strict aliasing rules
- add byteshift helpers
- main: set TCMUR_DEV_FLAG_IS_OPEN on successfully added devices
- libtcmu: fix cfgstring len
- README: fix highlighting the targetcli command looks like a title
- log/file_output: fix and clean up the file output callout
- logdir: add tcmu_logdir_check helper support
- alua:fix untrack when implicit transition not success
- runner: user tcmu_xxx helpers instead of printf
- main: fix crash by --tcmu-log-dir/-l
- tcmu-runner: make the max open files setup as operational
- tcmu-runner: increase max open file descriptors limit
- alua: to make sure the device failover is supported when alau access type is Implicit
- tcmur: fix rhandler->lock NULL pointer dereference
- alua: add failover_is_supported helper support
- rbd: misc fixes
- rbd: Add timers check for cluster osd
- tcmu-runner:fix the exception return value for function tcmu_get_alua_grps()
- tcmu:double free tgt_port in tcmu_get_alua_grp
- tcmu:fix return data length of rtpg response data
- .gitignore: update .gitignore
- runner: add errno info in err logs
- rbd: fix defined but not used warning
- cmake: add -Werror for CFLAGS
- rbd:fix comment in tcmu_rbd_cfg_desc
- runner: scsi debug info convert to tcmu_dev_dbg_scsi_cmd()
- alua: fix enabled_port NULL pointer dereference
- tcmur/unmap_split: reduce the unmap split logs output
- runner: fix main failure path
- libtcmu: merge resources release code
- rbd: Add the header explicitly when needed

* Tue Aug 15 2017 Jason Dillaman <dillaman@redhat.com> 1.3.0-0.2.2
- Depend on librbd1 packages from epoch 2

* Tue Aug 15 2017 Jason Dillaman <dillaman@redhat.com> 1.3.0-0.2.1
- Restrict to x86_64 arch

* Tue Aug 15 2017 Jason Dillaman <dillaman@redhat.com> 1.3.0-0.2
- Rebased to 1.3.0 RC2

